#!/bin/bash

# (C) 2012 Peter Senft <peter@rfc3092.net>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Parameters:
# - name of process
# - critical time
# - critical time unit (m - minute / h - hour / d - day / w - week
# - warning time
# - warning time unit (m - minute / h - hour / d - day / w - week

# Version History:
# 1.0 - initial release

check_process () {

	SEARCH="${1}"
	LIMIT=${2}
	LIMITUNIT="${3}"
	WARN=${4}
	WARNUNIT="${5}"

	case "${LIMITUNIT}" in
		m)
			(( LIMIT *= 60 ))
			;;
		h)
			(( LIMIT *= 60 * 60 ))
			;;
		d)
			(( LIMIT *= 24 * 60 * 60 ))
			;;
		w)
			(( LIMIT *= 7 * 24 * 60 * 60 ))
			;;
		*)
			;;
	esac
		
	case "${WARNUNIT}" in
		m)
			(( WARN *= 60 ))
			;;
		h)
			(( WARN *= 60 * 60 ))
			;;
		d)
			(( WARN *= 24 * 60 * 60 ))
			;;
		w)
			(( WARN *= 7 * 24 * 60 * 60 ))
			;;
		*)
			;;
	esac
	
	MSG=""
	LIMITMSG=" Processes running too long:"
	WARNMSG=" Processes running longer than expected:"
	LIMITCOUNT=0
	WARNCOUNT=0

	for i in `ps aux | grep ${SEARCH} | grep -v grep | grep -v bash | awk ' { print $2 } '`; do

		PROCESSTIME=`stat -c %Z /proc/${i} 2>&1`

		if [ $? -ne 0 ]; then
			continue
		fi

		SYSTEMTIME=`date +%s`
		RUNTIME=$((${SYSTEMTIME} - ${PROCESSTIME}))

		if [ ${RUNTIME} -gt ${LIMIT} ]; then
			(( LIMITCOUNT++ ))
			LIMITMSG="${LIMITMSG} ${i}"
		elif [ ${RUNTIME} -gt ${WARN} ]; then
			(( WARNCOUNT++ ))
			WARNMSG="${WARNMSG} ${i}"
		fi
	done

	if [ ${LIMITCOUNT} -eq 0 ] && [ ${WARNCOUNT} -eq 0 ]; then
		MSG="OK - No '${SEARCH}' process running too long."
	elif [ ${LIMITCOUNT} -eq 0 ]; then
		MSG="WARNING - ${WARNCOUNT} '${SEARCH}' processes running longer than expected."
	elif [ ${WARNCOUNT} -eq 0 ]; then
		MSG="CRITICAL - ${LIMITCOUNT} '${SEARCH}' processes running too long and should be terminated."
	fi

	if [ ${WARNCOUNT} -gt 0 ]; then
		MSG="${MSG} ${WARNMSG}"
	elif [ ${LIMITCOUNT} -gt 0 ]; then
		MSG="${MSG} ${LIMITMSG}"
	fi

	echo ${MSG}

	if [ ${WARNCOUNT} -gt 0 ]; then
		return 1
	elif [ ${LIMITCOUNT} -gt 0 ]; then
		return 2
	fi

	return 0
}

usage () {
	BASENAME=`basename $0`
	cat << EOF
Usage: ${BASENAME} -p <process> -c <crit> -w <warn> [-h]

OPTIONS:
  -h  Show this message
  -p  Name of process to be checked (i.e. "init")
  -c  Critical time. Allowed time units:
      s - seconds / m - minutes / h - hours / d - days / w - weeks
      Examples: 72h, 1w, 10s
  -w  Warning time. Allowed time units:
      s - seconds / m - minutes / h - hours / d - days / w - weeks
      Examples: 72h, 1w, 10s
EOF
	exit 3
}

SEARCH=
LIMITPARM=
LIMIT=
LIMITUNIT=
WARNPARM=
WARN=
WARNUNIT=

while getopts "hp:w:c:" OPTION
do
	case ${OPTION} in
		h)
			usage
			;;
		p)
			SEARCH=${OPTARG}
			;;
		w)
			WARNPARM=${OPTARG}
			;;
		c)
			LIMITPARM=${OPTARG}
			;;
		*)
			usage
			;;
	esac
done

if [ -z ${SEARCH} ] || [ -z ${WARNPARM} ] || [ -z ${LIMITPARM} ]; then
	usage
fi

LIMIT=`expr match "${LIMITPARM}" '^\([1-9][0-9]*\).*$'`
LIMITUNIT=`expr match "${LIMITPARM}" '^.*\([smhdw]\)$'`
WARN=`expr match "${WARNPARM}" '^\([1-9][0-9]*\).*$'`
WARNUNIT=`expr match "${WARNPARM}" '^.*\([smhdw]\)$'`

if [ -z ${LIMIT} ] || [ -z ${LIMITUNIT} ] || [ -z ${WARN} ] || [ -z ${WARNUNIT} ]; then
	usage
fi

check_process "${SEARCH}" "${LIMIT}" "${LIMITUNIT}" "${WARN}" "${WARNUNIT}"
exit $?
